@@ -128,8 +128,10 @@ PyTypeObject JSObjectProxyType = {
 PyTypeObject JSStringProxyType = {
   .tp_name = PyUnicode_Type.tp_name,
   .tp_basicsize = sizeof(JSStringProxy),
+  .tp_dealloc = (destructor)JSStringProxyMethodDefinitions::JSStringProxy_dealloc,
   .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_UNICODE_SUBCLASS,
-  .tp_doc = PyDoc_STR("Javascript String value"),
+  .tp_doc = PyDoc_STR("Javascript String proxy"),
+  .tp_methods = JSStringProxy_methods,
   .tp_base = &PyUnicode_Type
 };
 
@@ -388,7 +390,7 @@ static PyObject *eval(PyObject *self, PyObject *args) {
 
     if (getEvalOption(evalOptions, "filename", &s)) options.setFile(s);
     if (getEvalOption(evalOptions, "lineno", &l)) options.setLine(l);
-    if (getEvalOption(evalOptions, "column", &l)) options.setColumn(l);
+    if (getEvalOption(evalOptions, "column", &l)) options.setColumn(JS::ColumnNumberOneOrigin(l));
     if (getEvalOption(evalOptions, "mutedErrors", &b)) options.setMutedErrors(b);
     if (getEvalOption(evalOptions, "noScriptRval", &b)) options.setNoScriptRval(b);
     if (getEvalOption(evalOptions, "selfHosting", &b)) options.setSelfHostingMode(b);
@@ -419,7 +421,7 @@ static PyObject *eval(PyObject *self, PyObject *args) {
 
   // compile the code to execute
   JS::RootedScript script(GLOBAL_CX);
-  JS::Rooted<JS::Value> *rval = new JS::Rooted<JS::Value>(GLOBAL_CX);
+  JS::Rooted<JS::Value> rval(GLOBAL_CX);
   if (code) {
     JS::SourceText<mozilla::Utf8Unit> source;
     const char *codeChars = PyUnicode_AsUTF8(code);
@@ -440,27 +442,17 @@ static PyObject *eval(PyObject *self, PyObject *args) {
   }
 
   // execute the compiled code; last expr goes to rval
-  if (!JS_ExecuteScript(GLOBAL_CX, script, rval)) {
+  if (!JS_ExecuteScript(GLOBAL_CX, script, &rval)) {
     setSpiderMonkeyException(GLOBAL_CX);
     return NULL;
   }
 
   // translate to the proper python type
-  PyObject *returnValue = pyTypeFactory(GLOBAL_CX, *rval);
+  PyObject *returnValue = pyTypeFactory(GLOBAL_CX, rval);
   if (PyErr_Occurred()) {
     return NULL;
   }
 
-  // TODO: Find a way to root strings for the lifetime of a proxying python string
-  js::ESClass cls = js::ESClass::Other;   // placeholder if `rval` is not a JSObject
-  if (rval->isObject()) {
-    JS::GetBuiltinClass(GLOBAL_CX, JS::RootedObject(GLOBAL_CX, &rval->toObject()), &cls);
-  }
-
-  if (!(rval->isString() || cls == js::ESClass::String)) {   // rval may be a string which must be kept alive.
-    delete rval;
-  }
-
   if (returnValue) {
     return returnValue;
   }
@@ -480,6 +472,11 @@ static PyObject *waitForEventLoop(PyObject *Py_UNUSED(self), PyObject *Py_UNUSED
   return PyObject_CallMethod(waiter, "wait", NULL);
 }
 
+static PyObject *closeAllPending(PyObject *Py_UNUSED(self), PyObject *Py_UNUSED(_)) {
+  if (!PyEventLoop::AsyncHandle::cancelAll()) return NULL;
+  Py_RETURN_NONE;
+}
+
 static PyObject *isCompilableUnit(PyObject *self, PyObject *args) {
   PyObject *item = PyTuple_GetItem(args, 0);
   if (!PyUnicode_Check(item)) {
@@ -489,15 +486,17 @@ static PyObject *isCompilableUnit(PyObject *self, PyObject *args) {
 
   const char *bufferUtf8 = PyUnicode_AsUTF8(item);
 
-  if (JS_Utf8BufferIsCompilableUnit(GLOBAL_CX, *global, bufferUtf8, strlen(bufferUtf8)))
+  if (JS_Utf8BufferIsCompilableUnit(GLOBAL_CX, *global, bufferUtf8, strlen(bufferUtf8))) {
     Py_RETURN_TRUE;
-  else
+  } else {
     Py_RETURN_FALSE;
+  }
 }
 
 PyMethodDef PythonMonkeyMethods[] = {
   {"eval", eval, METH_VARARGS, "Javascript evaluator in Python"},
   {"wait", waitForEventLoop, METH_NOARGS, "The event-loop shield. Blocks until all asynchronous jobs finish."},
+  {"stop", closeAllPending, METH_NOARGS, "Cancel all pending event-loop jobs."},
   {"isCompilableUnit", isCompilableUnit, METH_VARARGS, "Hint if a string might be compilable Javascript"},
   {"collect", collect, METH_VARARGS, "Calls the Spidermonkey garbage collector"},
   {NULL, NULL, 0, NULL}
@@ -552,8 +551,6 @@ PyMODINIT_FUNC PyInit_pythonmonkey(void)
 
   JS::RealmCreationOptions creationOptions = JS::RealmCreationOptions();
   JS::RealmBehaviors behaviours = JS::RealmBehaviors();
-  creationOptions.setWeakRefsEnabled(JS::WeakRefSpecifier::EnabledWithoutCleanupSome); // enable FinalizationRegistry
-  creationOptions.setIteratorHelpersEnabled(true);
   JS::RealmOptions options = JS::RealmOptions(creationOptions, behaviours);
   static JSClass globalClass = {"global", JSCLASS_GLOBAL_FLAGS, &JS::DefaultGlobalClassOps};
   global = new JS::RootedObject(GLOBAL_CX, JS_NewGlobalObject(GLOBAL_CX, &globalClass, nullptr, JS::FireOnNewGlobalHook, options));
@@ -565,14 +562,19 @@ PyMODINIT_FUNC PyInit_pythonmonkey(void)
   JS::RootedObject debuggerGlobal(GLOBAL_CX, JS_NewGlobalObject(GLOBAL_CX, &globalClass, nullptr, JS::FireOnNewGlobalHook, options));
   {
     JSAutoRealm r(GLOBAL_CX, debuggerGlobal);
-    JS_DefineProperty(GLOBAL_CX, debuggerGlobal, "mainGlobal", *global, JSPROP_READONLY);
     JS_DefineDebuggerObject(GLOBAL_CX, debuggerGlobal);
   }
+  {
+    JSAutoRealm r(GLOBAL_CX, *global);
+    JS::Rooted<JS::PropertyDescriptor> desc(GLOBAL_CX, JS::PropertyDescriptor::Data(
+      JS::ObjectValue(*debuggerGlobal)
+    ));
+    JS_WrapPropertyDescriptor(GLOBAL_CX, &desc);
+    JS_DefineUCProperty(GLOBAL_CX, *global, u"debuggerGlobal", 14, desc);
+  }
 
   autoRealm = new JSAutoRealm(GLOBAL_CX, *global);
 
-  JS_DefineProperty(GLOBAL_CX, *global, "debuggerGlobal", debuggerGlobal, JSPROP_READONLY);
-
   // XXX: SpiderMonkey bug???
   // In https://hg.mozilla.org/releases/mozilla-esr102/file/3b574e1/js/src/jit/CacheIR.cpp#l317, trying to use the callback returned by `js::GetDOMProxyShadowsCheck()` even it's unset (nullptr)
   // Temporarily solved by explicitly setting the `domProxyShadowsCheck` callback here
